/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
/*
__IMPORT__
	(
	"chrome://mozlib/content/lib/js/digest/",
	"generator.js",
	"md5.js"
	);
__IMPORT__
	(
	"chrome://mozlib/content/lib/js/rdfapi/",
	"nsrdf.js",
	"node.js",
	"resource.js",
	"literal.js",
	"statement.js",
	"statementObserver.js",
	"setmodel.js",
	"setstore.js",
	"storeObserver.js",
	"collector.js",
	"iterator.js"
	);
*/


//@@@@@@@@@@@@@@@@
function RDFAPI_Model		//<c><e>RDFAPI_Store<a>protected<d>RDF model class
(
_storeURL,		//<p>string<d>URL or null for in-memory model
_storage		//<p>int<d>Storage mode according to loading process
){
const ASSERTING = true;
//================
this.RDFAPI_Model = function		//<m>void<a>private<d>class constructor
(
storeURL,	//<p>string<d>URL or empty string for in-memory model
storage		//<p>int<d>Storage mode according to loading process
){
this.className	= 'RDFAPI_Model';
this.storeURL	= storeURL;
this.storage	= storage;
//i12n
this.URI		= '',
this.preserving	= true;
this.store		= new RDFAPI_Store(this.storeURL, this.storage);
this.setModel	= new RDFAPI_SetModel();
}//</m>RDFAPI_Model



//================
this.set_connected = function		//<m>string<a>protected<d>Set up connected member
(){
this.connected = true;
//__ASSERT__.isTrue(this.connected, 'set_connected: this.connected == ' + ASSERTING);
}//</m>set_connected



//================
this.set_preserving = function		//<m>boolean<a>public<d>Set up preserving member
(
preserving			//<p>boolean<d>Replace previous assertion flag returning previous
){
var
/*boolean*/previous = this.preserving;
this.preserving = preserving;
return previous;
}//</m>set_preserving



//================
this.setSourceURI = function		//<m>void<a>public<d>Set up URI member
(
uri		//<p>string<d>Base URI
){
this.URI = uri || this.storeURL || (new DIGEST_MD5Generator()).getHex(new Date());
}//</m>setSourceURI



//================
this.setAddStatement = function		//<m>(boolean)int<a>public<d>Add a statement to the model
(
statement		//<p>RDFAPI_Statement<d>Statement to assert
){
return this.setStatement(statement, 'Assert');
}//</m>setAddStatement



//================
this.setRemoveStatement = function		//<m>(boolean)int<a>public<d>Remove a model statement
(
statement		//<p>RDFAPI_Statement<d>Statement to unassert
){
return this.setStatement(statement, 'Unassert');
}//</m>setRemoveStatement



//================
this.setModifyStatement = function		//<m>boolean(int)<a>public<d>Modify this statement in the model
(
statement,		//<p>RDFAPI_Statement<d>Statement to modify
replacement,	//<p>Object<d>Replacement for subject (move) or object (change) resource
moving			//<p>boolean<d>Move flag: move || change
){
try{
	replacement = statement.getFromSource(replacement, !moving);
	if(moving){
		this.getStore().Move(
			statement.getSubjectAsResource(),
			replacement.getAsResource(),
			statement.getPredicateAsResource(),
			statement.getObjectAsNode());
		statement.set_subject(replacement);
		}
	else{
		this.getStore().Change(
			statement.getSubjectAsResource(),
			statement.getPredicateAsResource(),
			statement.getObjectAsNode(),
			replacement[(replacement instanceof RDFAPI_Literal) ? 'getAsLiteral' : 'getAsResource']());
		statement.set_object(replacement);
		}
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.K.ERROR.NO_SUCCESS;
	}
}//</m>setModifyStatement



//================
this.setAddMarkStatementSet = function		//<m>boolean(int)<a>public<d>Add the statement to the marked statement set
(
statement,		//<p>RDFAPI_Statement<d>Statement to mark
preserving		//<p>boolean<d>Replace previous assertion flag: preserve || replace
){
try{
	return this.store.getMarkStore().Mark(
		statement.getSubjectAsResource(),
		statement.getPredicateAsResource(),
		statement.getObjectAsResource(),
		this.preserving) ?
			this.K.ERROR.NONE :
			this.K.ERROR.NO_SUCCESS;
	}
catch(e){
	return this.K.ERROR.NO_SUCCESS;
	}
}//</m>setAddMarkStatementSet



//================
this.setRemoveMarkStatementSet = function		//<m>(boolean)int<a>public<d>Remove statements in the marked statement set
(){
try{
	this.store.getMarkStore().Sweep();
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.K.ERROR.NO_SUCCESS;
	}
}//</m>setRemoveMarkStatementSet



//================
this.setOut = function		//<m>boolean(int)<a>public<d>Flush model
(
url		//<p>String<d>Destination URL --optional
){
return this.store.setOut(url);
}//</m>setOut<d>&
/*
Delegate to store
*/



//================
this.setRemoveStatementSet = function		//<m>(boolean)int<a>public<d>Remove a collection of previously marked statement
(){
return this.store.setRemoveStatementSet();
}//</m>setRemoveStatementSet<d>&
/*
Delegate to store
*/



//================	FIXME: do we need this
this.setResourceRegistry = function		//<m>(boolean)int<a>public<d>Register or Unregister the resource
(
resource,			//<p>Object<d>Resource to compare
registering,		//<p>boolean<d>Register flag: register || unregister
replacing			//<p>boolean<d>Replace previously registered flag: replace || preserve
){
return this.store.setResourceRegistry(resource, registering, replacing);
}//</m>setResourceRegistry<d>&
/*
Delegate to store
*/



//================	FIXME: do we need this ?
this.setModelRegistry = function		//<m>(boolean)int<a>public<d>Register or unregister the model
(
registering,		//<p>boolean<d>Register or unregister
replacing			//<p>boolean=false<d>Replace previous model when registering
){
return this.store.setModelRegistry(registering, replacing);
}//</m>setModelRegistry<d>&
/*
Delegate to store
*/



//================
this.setObserver = function		//<m>(boolean)int<a>public<d>Set up an observer of the model
(
observer,		//<p>RDFAPI_StatementObserver<d>Observer
registering		//<p>boolean<d>Register or unregister the observer
){
return this.store.setObserver(observer, registering);
}//</m>setObserver<d>&
/*
Delegate to store
*/



//================
this.setHandleObserverDelegate = function		//<m>void<a>private<d>Handles statement observer events
(
notifier		//<p>Object<d>Observer arguments
){
//TODO
}//</m>setHandleObserverDelegate



//================
this.getSourceURI = function		//<m>string<a>public<d>Get URI member
(){
return this.URI || this.setSourceURI();
}//</m>get_connected<d>&
/*
If URI member has not been previously set, a default value from setSourceURI is retrieved
*/



//================
this.get_storeURL = function		//<m>string<a>protected<d>Get storeURL member
(){
return this.storeURL;
}//</m>get_storeURL



//================
this.get_storage = function		//<m>int<a>protected<d>Get storage member
(){
return this.storage;
}//</m>get_storage



//================
this.getStore = function		//<m>int<a>protected<d>Get store
(){
return this.store.get_store();
}//</m>getStore



//================
this.getContainer = function		//<m>Object<a>public<d>Get a model container
(
resource,	//<p>RDFAPI_Resource<d>Container resource
type		//<p>int<d>Container type
){
return new RDFAPI_Collector(resource, type, this.getStore());
}//</m>getContainer



//================
this.getInstance = function		//<m>int<a>public<d>Get a new (empty) model
(){
return new RDFAPI_Model(null, this.store.K.MEMORY_SYNC_STORAGE);
}//</m>getInstance



//================
this.getDuplicata = function		//<m>int<a>public<d>Get a duplicata of the model
(){
var
modelClass = eval(this.className),
/*RDFAPI_Model*/model = new modelClass(null, this.store.K.MEMORY_SYNC_STORAGE),		//FIXME
/*RDFAPI_Statement*/statementSet = this.getStatementSet();
for(var i=0,m=statementSet.length;i<m;i++) model.setAddStatement(statementSet[i]);
return model;
}//</m>getDuplicata



//================
this.getSize = function		//<m>int<a>public<d>Get number of resources in the model
(){
try{
	var
	/*int*/count,
	/*Object*/enumerator = this.getSubjectSet();
	for(count=0; enumerator.hasMoreElements(); count++) enumerator.getNext();
	return count;
	}
catch(e){
	return null;
	}
}//</m>getSize



//================
this.getSubjectSet = function		//<m>Object<a>Get an enumeration of all subject resources in store
(){
return this.store.getSubjectSet();
}//</m>getSubjectSet<d>&
/*
Delegate to store
*/



//================
this.getArcSet = function		//<m>Object<a>public<d>Get an enumeration of arcs referring to resource
(
resource,		//<p>Object<d>Resource to test
outcoming		//<p>boolean<d>Direction flag: out || in
){
return this.store.getArcSet(resource, outcoming);
}//</m>getArcSet<d>&
/*
Delegate to store
*/



//================
this.getStatementSet = function		//<m>RDFAPI_Statement[]<a>public<d>Get an array of all statements in store
(){
return this.store.getStatementSet();
}//</m>getStatementSet<d>&
/*
Delegate to store
*/



//================
this.getSubjectStatementSet = function		//<m>Object<a>public<d>Get an enumeration of all statements of a subject
(
subject		//<p>Object<d>Subject nsIRDFResource
){
return this.store.getSubjectStatementSet(subject);
}//</m>getSubjectStatementSet<d>&
/*
Delegate to store
*/



//================
this.getMatchingStatementSet = function		//<m>RDFAPI_NSIModel<a>public<d>Get a new model containing statements matching arguments
(
subject,		//<p>RDFAPI_Resouce<d>Statement subject -- can be a RegExp
predicate,		//<p>RDFAPI_Resouce<d>Statement predicate -- can be a RegExp
object,			//<p>Object<d>Statement object -- can be a RegExp
excluding		//<p>boolean=false<d>Exclusion flag: NOT AND (XOR) || AND
){
if(!subject && !predicate && !object) return this.getDuplicata();
	///*PROFILE*/var start = new Date();
var
modelClass = eval(this.className),
/*RDFAPI_NSIModel*/model = new modelClass(null, this.store.K.MEMORY_SYNC_STORAGE);
//__ASSERT__.isNotEqual(0, this.getSize(), 'getMatchingStatementSet: getSize', ASSERTING);
if(this.getSize()){
	var
	/*RDFAPI_Statement*/statement,
	/*RDFAPI_Statement[]*/statementSet = this.getStatementSet();
	//__ASSERT__.isNotEqual(0, statementSet.length, 'getMatchingStatementSet: length', ASSERTING);
	for(var i=0,m=statementSet.length;i<m;i++){
		statement = statementSet[i];
		if(this.testStatementMatch(statement, subject, predicate, object, excluding)) model.setAddStatement(statement);
		}	//---for
	}
	///*PROFILE*/Stdout.setOut('getMatchingStatementSet profiling: %N', (((new Date()) - start) / 1000));
return model;
}//</m>getMatchingStatementSet



//================
this.getBranch = function		//<m>RDFAPI_Model<a>public<d>Get a sub-model of this one containing nodes reachable from atNode
(
resource,		//<p>RDFAPI_Node<d>Sub-model root resource
branch			//<p>RDFAPI_Model<d>Sub-model agregator
){
//__ASSERT__.warn('getBranch: resource URI == ' + resource.getURI(), ASSERTING);
if(resource instanceof RDFAPI_Resource){
	var
	/*RDFAPI_Resource*/object,
	/*RDFAPI_Model*/model = this.getMatchingStatementSet(resource, null, null),
	/*RDFAPI_Statement[]*/statementSet = model.getStatementSet();
	/*RDFAPI_Model*/branch = this.setModel.getUnion(branch, model);
	for(var i=0,m=statementSet.length;i<m;i++){
		object = statementSet[i].getObject();
		//__ASSERT__.warn('getBranch: object URI == ' + object.getURI(), ASSERTING);
		if(object instanceof RDFAPI_Resource) branch = this.getBranch(object, branch);
		}	//---for
	}
return branch;
}//</m>getBranch<d>&
/*
Recursive method
*/



//================
this.testEmpty = function		//<m>boolean<a>public<d>Check whether model is empty
(){
return this.getSize() == 0;
}//</m>testEmpty



//================
this.testOccurence = function		//<m>boolean<a>public<d>Check whether model contains the statement
(
statement		//<p>RDFAPI_Statement<d>Statement to check
){
try{
	return this.getStore().HasAssertion(
		statement.getSubjectAsResource(),
		statement.getPredicateAsResource(),
		statement.getObjectAsNode(),
		this.preserving);
	}
catch(e){
	return false;
	}
}//</m>testOccurence



//================
this.testPredicate = function		//<m>boolean<a>public<d>Check whether a predicate has the resource as source or target
(
resource,		//<p>RDFAPI_Resource<d>Resource to test
predicate,		//<p>RDFAPI_Resource<d>Predicate as RDFAPI_Resource or nsIRDFResource
outcoming		//<p>boolean<d>Direction flag: out || in
){
try{
	var
	predicate = (predicate instanceof RDFAPI_Resource) ? predicate.getAsResource() : predicate;
	return this.getStore()[outcoming ? 'hasArcIn' : 'hasArcOut'](resource, predicate);
	}
catch(e){
	return false;
	}
}//</m>testPredicate



//================
this.testEquality = function		//<m>boolean<a>public<d>Check whether the model is the same as
(
model		//<p>RDFAPI_Model<d>Model to compare
){
try{
	//NOT IMPLEMENTED .......................
	}
catch(e){
	return false;
	}
}//</m>testEquality



//================
this.testCollector = function		//<m>boolean<a>public<d>Check whether resource is a container
(
resource		//<p>RDFAPI_Resource<d>Resource to test
){
return this.store.testCollector(resource);
}//</m>testCollector<d>&
/*
Delegate to store
*/



this.K =
	{
	AT_PREDICATE:		0,
	AT_SUBJECT:			1,
	AT_OBJECT:			2,

	ERROR:
		{
		//NON FATAL
		NONE:			0,
		NO_SUCCESS:		1,
		//FATAL         
		FATAL:			0xFF		//end of error codes
		}
	};



/* STANFORD RDF API
==============================================

//this.setSourceURI
this.add		= this.setAddStatement;
this.remove		= this.setRemoveStatement;
//this.getSourceURI
this.size		= this.getSize;
this.elements	= this.getStatementSet;
this.find		= this.getMatchingStatementSet;
this.create		= this.getInstance;
this.duplicate	= this.getDuplicata;
this.isEmpty	= this.testEmpty;
this.contains	= this.testOccurence;

==============================================
*/
this.RDFAPI_Model(_storeURL, _storage);
}//</c>RDFAPI_Model

