/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
//@@@@@@@@@@@@@@@@
function XPI_Installer		//<c><a>public<d>XPI installer class
(
_label,			//<p>string<d>Application package name (all lowercase required)
_propPath		//<p>string<d>L10n property file path
){
//================
this.XPI_Installer = function		//<m>void<a>private<d>class constructor
(
label,			//<p>string<d>Application package name (all lowercase required)
propPath		//<p>string<d>L10n property file path
){
this.className	= 'XPI_Installer';
this.label		= label;
this.propPath	= propPath;
this.setIn();
}//</m>XPI_Installer



//================
this.setIn = function		//<m>void<a>private<d>Instance i12n
(){
this.installer	= Install;
this.l10nSet	= null;		//i12n delayed: loadResources can NOT be called before initInstall method
}//</m>setIn



//================
this.setUp = function		//<m>int<a>public<d>Set up installation parameters
(
name,			//<p>string<d>Application display name
version,		//<p>string<d>Application version
monitor			//<p>Object<d>Installation monitor singleton
){
var
/*string*/code = this.installer.initInstall(name, this.label, version);
this.l10nSet = this.installer.loadResources(this.propPath);
if(!this.l10nSet) return this.setAbort(this.K.PROPERTY_ACCESS, this.K.ERROR.PROPERTY_ACCESS);
if(code != this.K.ERROR.NONE) return this.setAbort(code);
this.setOutLog(this.K.INSTALL_START, new Date(), name, this.label, version);
if((this.testSpace() == this.K.ERROR.NONE) && (monitor.setUp(this) == this.K.ERROR.NONE)) return this.setInstall();
}//</m>setUp



//================
this.setInstall = function		//<m>int<a>private<d>Proceed to installation
(){
this.installer.performInstall();
this.setOutLog(this.K.INSTALL_END);
return this.K.ERROR.NONE;
}//</m>setInstall



//================
this.setAbort = function		//<m>int<a>public<d>Abort installation process
(
code,		//<p>int<d>Error code
logging		//<p>boolean<d>Log method flag: Install::logComment || XPI_Installer::setOutLog
){
if(logging) this.installer.logComment(code);
else this.setOutLog(this.K.INSTALL_ERROR, code);
this.installer.cancelInstall(code);
return this.K.ERROR.NO_SUCCESS;
}//</m>setAbort



//================
this.setOutLog = function		//<m>void<a>private<d>Log log to the installation log file
(
property		//<p>string<d>Property for l10n
//propertySet	//<p>string[]<d>Run time property set to insert in property text
){
var
/*string*/log = '',
/*string[]*/logSet = this.l10nSet[property].split('%s');
if(logSet) for(var i=1,m=logSet.length+1;i<m;i++) log += logSet[i-1] + ((arguments[i] != undefined) ? arguments[i] : '');
else log = this.l10nSet[property];
this.installer.logComment(log);
}//</m>setOutLog<d>&
/*
Modified from sprintf function
*/



//================
this.testSpace = function		//<m>boolean(int)<a>private<d>Check whether available disk space is OK to perform install
(
dirPath,		//<p>string<d>Directory path
required		//<p>int<d>Space required in bytes
){
var
/*int*/available = parseInt(fileGetDiskSpaceAvailable(dirPath) / 1024);
if(available > required) return this.K.ERROR.NONE;
this.setOutLog(this.K.INSTALL_SPACE, dirPath, required, available);
this.setAbort(this.K.ERROR.DISK_SPACE);
return this.K.ERROR.NO_SUCCESS;
}//</m>testSpace



this.K =
	{
	PROPERTY_ACCESS:		'INSTALLATION ERROR: Localization file not found.',
	INSTALL_ERROR:			'xpi.install.error',
	INSTALL_START:			'xpi.install.start',
	INSTALL_END:			'xpi.install.end',
	INSTALL_SPACE:			'xpi.install.space',

	ERROR:
		{
		//NON FATAL
		NONE:				0,
		NO_SUCCESS:			1,

		//FATAL
		DISK_SPACE:			-235
		PROPERTY_ACCESS:	-238,
		FATAL:				0xFF		//end of error codes
		}
	};



this.XPI_Installer(_label _propPath);
}//</c>XPI_Installer



